@@ -20,6 +20,8 @@ from paste.script.pluginlib import find_egg_info_dir
 
 import pylons.util as util
 
+__all__ = ['ControllerCommand', 'RestControllerCommand', 'ShellCommand']
+
 def can_import(name):
     """Attempt to __import__ the specified package/module, returning True when
     succeeding, otherwise False"""
@@ -29,13 +31,33 @@ def can_import(name):
     except ImportError:
         return False
 
+
+def is_minimal_template(package):
+    """Determine if the specified Pylons project (package) uses the Pylons
+    Minimal Tempalte"""
+    minimal_template = False
+    try:
+        # Check if PACKAGE.lib.base exists
+        __import__(package + '.lib.base')
+    except ImportError, ie:
+        if 'No module named lib.base' in str(ie):
+            minimal_template = True
+    except:
+        # PACKAGE.lib.base exists but throws an error
+        pass
+    return minimal_template
+
+
 def validate_name(name):
     """Validate that the name for the controller isn't present on the
     path already"""
     if not name:
         # This happens when the name is an existing directory
         raise BadCommand('Please give the name of a controller.')
-    if can_import(name):
+    # 'setup' is a valid controller name, but when paster controller is ran
+    # from the root directory of a project, importing setup will import the
+    # project's setup.py causing a sys.exit(). Blame relative imports
+    if name != 'setup' and can_import(name):
         raise BadCommand(
             "\n\nA module named '%s' is already present in your "
             "PYTHON_PATH.\nChoosing a conflicting name will likely cause "
@@ -47,22 +69,23 @@ def validate_name(name):
             % (name, name, name, name))
     return True
 
+
 class ControllerCommand(Command):
     """Create a Controller and accompanying functional test
-    
+
     The Controller command will create the standard controller template
     file and associated functional test to speed creation of controllers.
-    
+
     Example usage::
-    
+
         yourproj% paster controller comments
         Creating yourproj/yourproj/controllers/comments.py
         Creating yourproj/yourproj/tests/functional/test_comments.py
-    
+
     If you'd like to have controllers underneath a directory, just include
     the path as the controller name and the necessary directories will be
     created for you::
-    
+
         yourproj% paster controller admin/trackback
         Creating yourproj/controllers/admin
         Creating yourproj/yourproj/controllers/admin/trackback.py
@@ -70,13 +93,13 @@ class ControllerCommand(Command):
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
-    
+
     min_args = 1
     max_args = 1
     group_name = 'pylons'
-    
+
     default_verbosity = 3
-    
+
     parser = Command.standard_parser(simulate=True)
     parser.add_option('--no-test',
                       action='store_true',
@@ -92,26 +115,23 @@ class ControllerCommand(Command):
                 name, directory = file_op.parse_path_name_args(self.args[0])
             except:
                 raise BadCommand('No egg_info directory was found')
-            
+
             # Check the name isn't the same as the package
             base_package = file_op.find_dir('controllers', True)[0]
             if base_package.lower() == name.lower():
                 raise BadCommand(
                     'Your controller name should not be the same as '
-                    'the package name %r.'% base_package
-            )
+                    'the package name %r.' % base_package)
             # Validate the name
             name = name.replace('-', '_')
             validate_name(name)
-            
-            # Check to see if PROJ.lib.base exists
-            try:
-                __import__(base_package + '.lib.base')
-                importstatement = "from %s.lib.base import *" % base_package
-            except:
-                # Assume its the minimal template
+
+            # Determine the module's import statement
+            if is_minimal_template(base_package):
                 importstatement = "from %s.controllers import *" % base_package
-            
+            else:
+                importstatement = "from %s.lib.base import *" % base_package
+
             # Setup the controller
             fullname = os.path.join(directory, name)
             controller_name = util.class_name_from_module_name(
@@ -119,11 +139,12 @@ class ControllerCommand(Command):
             if not fullname.startswith(os.sep):
                 fullname = os.sep + fullname
             testname = fullname.replace(os.sep, '_')[1:]
-            file_op.template_vars.update({'name': controller_name,
-                                          'fname': os.path.join(directory, name),
-                                          'importstatement': importstatement})
+            file_op.template_vars.update(
+                {'name': controller_name,
+                 'fname': os.path.join(directory, name),
+                 'importstatement': importstatement})
             file_op.copy_file(template='controller.py_tmpl',
-                         dest=os.path.join('controllers', directory), 
+                         dest=os.path.join('controllers', directory),
                          filename=name)
             if not self.options.no_test:
                 file_op.copy_file(template='test_controller.py_tmpl',
@@ -135,29 +156,30 @@ class ControllerCommand(Command):
             msg = str(sys.exc_info()[1])
             raise BadCommand('An unknown error occurred. %s' % msg)
 
+
 class RestControllerCommand(Command):
     """Create a REST Controller and accompanying functional test
-    
+
     The RestController command will create a REST-based Controller file for use
     with the map.resource REST-based dispatching. This template includes the
     methods that map.resource dispatches to in addition to doc strings for
     clarification on when the methods will be called.
-    
-    The first argument should be the singular form of the REST resource. The 
+
+    The first argument should be the singular form of the REST resource. The
     second argument is the plural form of the word. If its a nested controller,
     put the directory information in front as shown in the second example
     below.
-    
+
     Example usage::
-    
+
         yourproj% paster restcontroller comment comments
         Creating yourproj/yourproj/controllers/comments.py
         Creating yourproj/yourproj/tests/functional/test_comments.py
-    
+
     If you'd like to have controllers underneath a directory, just include
     the path as the controller name and the necessary directories will be
     created for you::
-    
+
         yourproj% paster restcontroller admin/tracback admin/trackbacks
         Creating yourproj/controllers/admin
         Creating yourproj/yourproj/controllers/admin/trackbacks.py
@@ -165,13 +187,13 @@ class RestControllerCommand(Command):
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
-    
+
     min_args = 2
     max_args = 2
     group_name = 'pylons'
-    
+
     default_verbosity = 3
-    
+
     parser = Command.standard_parser(simulate=True)
     parser.add_option('--no-test',
                       action='store_true',
@@ -190,19 +212,24 @@ class RestControllerCommand(Command):
                     file_op.parse_path_name_args(self.args[1])
             except:
                 raise BadCommand('No egg_info directory was found')
-            
+
             # Check the name isn't the same as the package
             base_package = file_op.find_dir('controllers', True)[0]
             if base_package.lower() == pluralname.lower():
                 raise BadCommand(
                     'Your controller name should not be the same as '
-                    'the package name %r.'% base_package
-            )
+                    'the package name %r.'% base_package)
             # Validate the name
             for name in [singularname, pluralname]:
                 name = name.replace('-', '_')
                 validate_name(name)
 
+            # Determine the module's import statement
+            if is_minimal_template(base_package):
+                importstatement = "from %s.controllers import *" % base_package
+            else:
+                importstatement = "from %s.lib.base import *" % base_package
+
             # Setup the controller
             fullname = os.path.join(pluraldirectory, pluralname)
             controller_name = util.class_name_from_module_name(
@@ -210,11 +237,11 @@ class RestControllerCommand(Command):
             if not fullname.startswith(os.sep):
                 fullname = os.sep + fullname
             testname = fullname.replace(os.sep, '_')[1:]
-            
+
             nameprefix = ''
             if pluraldirectory:
                 nameprefix = pluraldirectory.replace(os.path.sep, '_') + '_'
-            
+
             controller_c = ''
             if nameprefix:
                 controller_c = ", controller='%s', \n\t" % \
@@ -223,7 +250,7 @@ class RestControllerCommand(Command):
                     (pluraldirectory, pluraldirectory)
             command = "map.resource('%s', '%s'%s)\n" % \
                 (singularname, pluralname, controller_c)
-            
+
             file_op.template_vars.update(
                 {'classname': controller_name,
                  'pluralname': pluralname,
@@ -232,16 +259,16 @@ class RestControllerCommand(Command):
                  'nameprefix': nameprefix,
                  'resource_command': command.replace('\n\t', '\n%s#%s' % \
                                                          (' '*4, ' '*9)),
-                 'fname': os.path.join(pluraldirectory, pluralname)}
-            )
-            
+                 'fname': os.path.join(pluraldirectory, pluralname),
+                 'importstatement': importstatement})
+
             resource_command = ("\nTo create the appropriate RESTful mapping, "
                                 "add a map statement to your\n")
             resource_command += ("config/routing.py file near the top like "
                                  "this:\n\n")
             resource_command += command
             file_op.copy_file(template='restcontroller.py_tmpl',
-                         dest=os.path.join('controllers', pluraldirectory), 
+                         dest=os.path.join('controllers', pluraldirectory),
                          filename=pluralname)
             if not self.options.no_test:
                 file_op.copy_file(template='test_controller.py_tmpl',
@@ -257,25 +284,35 @@ class RestControllerCommand(Command):
 
 class ShellCommand(Command):
     """Open an interactive shell with the Pylons app loaded
-    
+
     The optional CONFIG_FILE argument specifies the config file to use for
     the interactive shell. CONFIG_FILE defaults to 'development.ini'.
-    
+
     This allows you to test your mapper, models, and simulate web requests
     using ``paste.fixture``.
-    
+
     Example::
-        
+
         $ paster shell my-development.ini
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
-    
+
     min_args = 0
     max_args = 1
     group_name = 'pylons'
-    
+
     parser = Command.standard_parser(simulate=True)
+    parser.add_option('-d', '--disable-ipython',
+                      action='store_true',
+                      dest='disable_ipython',
+                      help="Don't use IPython if it is available")
+
+    parser.add_option('-q',
+                      action='count',
+                      dest='quiet',
+                      default=0,
+                      help="Do not load logging configuration from the config file")
 
     def command(self):
         """Main command to create a new shell"""
@@ -290,24 +327,29 @@ class ShellCommand(Command):
                                   config_file))
         else:
             config_file = self.args[0]
-            
+
         config_name = 'config:%s' % config_file
         here_dir = os.getcwd()
         locs = dict(__name__="pylons-admin")
 
+        # XXX: Note, initializing CONFIG here is Legacy support. pylons.config
+        # will automatically be initialized and restored via the registry
+        # restorer along with the other StackedObjectProxys
         # Load app config into paste.deploy to simulate request config
+        # Setup the Paste CONFIG object, adding app_conf/global_conf for legacy
+        # code
         conf = appconfig(config_name, relative_to=here_dir)
-        conf = dict(app_conf=conf.local_conf,
-                    global_conf=conf.global_conf)
+        conf.update(dict(app_conf=conf.local_conf,
+                         global_conf=conf.global_conf))
         paste.deploy.config.CONFIG.push_thread_config(conf)
-        
+
         # Load locals and populate with objects for use in shell
         sys.path.insert(0, here_dir)
-        
+
         # Load the wsgi app first so that everything is initialized right
         wsgiapp = loadapp(config_name, relative_to=here_dir)
         test_app = paste.fixture.TestApp(wsgiapp)
-        
+
         # Query the test app to setup the environment
         tresponse = test_app.get('/_test_vars')
         request_id = int(tresponse.body)
@@ -341,69 +383,54 @@ class ShellCommand(Command):
                 found_base = can_import(base_module)
 
             if found_base:
-                models_package = pkg_name + '.models'
-                # The Minimal template lacks an official models package
-                has_models = can_import(models_package)
                 break
 
         if not found_base:
-            raise ImportError("Could not import base module. Are you sure this "
-                              "is a Pylons app?") 
+            raise ImportError("Could not import base module. Are you sure "
+                              "this is a Pylons app?")
 
         base = sys.modules[base_module]
         base_public = [__name for __name in dir(base) if not \
                        __name.startswith('_') or __name == '_']
         for name in base_public:
             locs[name] = getattr(base, name)
-        locs.update(
-            dict(
-                mapper=tresponse.pylons_config.map,
-                wsgiapp=wsgiapp,
-                app=test_app,
-            )
-        )
-        if has_models:
-            locs['model'] = sys.modules[models_package]
+        locs.update(dict(wsgiapp=wsgiapp, app=test_app))
+
+        mapper = tresponse.config.get('routes.map')
+        if mapper:
+            locs['mapper'] = mapper
 
         banner = "  All objects from %s are available\n" % base_module
         banner += "  Additional Objects:\n"
-        banner += "  %-10s -  %s\n" % ('mapper', 'Routes mapper object')
-        if has_models:
-            banner += "  %-10s -  %s\n" % ('model',
-                                           'Models from models package')
-        banner += "  %-10s -  %s\n" % ('wsgiapp', 
-            'This projects WSGI App instance')
-        banner += "  %-10s -  %s\n" % ('app', 
+        if mapper:
+            banner += "  %-10s -  %s\n" % ('mapper', 'Routes mapper object')
+        banner += "  %-10s -  %s\n" % ('wsgiapp',
+            "This project's WSGI App instance")
+        banner += "  %-10s -  %s\n" % ('app',
             'paste.fixture wrapped around wsgiapp')
 
+        if not self.options.quiet:
+            # Configure logging from the config file
+            self.logging_file_config(config_file)
+        
         try:
+            if self.options.disable_ipython:
+                raise ImportError()
+
             # try to use IPython if possible
             from IPython.Shell import IPShellEmbed
-            
-            shell = IPShellEmbed(user_ns=locs)
+
+            shell = IPShellEmbed(argv=self.args)
             shell.set_banner(shell.IP.BANNER + '\n\n' + banner)
             try:
-                shell()
+                shell(local_ns=locs, global_ns={})
             finally:
                 paste.registry.restorer.restoration_end()
         except ImportError:
             import code
-            
-            class CustomShell(code.InteractiveConsole):
-                """Custom shell class to handle raw input"""
-                def raw_input(self, *args, **kw):
-                    """Capture raw input in exception wrapping"""
-                    try:
-                        return code.InteractiveConsole.raw_input(
-                            self, *args, **kw)
-                    except EOFError:
-                        # In the future, we'll put our own override as needed 
-                        # to save models, TG style
-                        raise EOFError
-            
             newbanner = "Pylons Interactive Shell\nPython %s\n\n" % sys.version
             banner = newbanner + banner
-            shell = CustomShell(locals=locs)
+            shell = code.InteractiveConsole(locals=locs)
             try:
                 import readline
             except ImportError:
@@ -412,5 +439,3 @@ class ShellCommand(Command):
                 shell.interact(banner)
             finally:
                 paste.registry.restorer.restoration_end()
-
-__all__ = ['ControllerCommand', 'RestControllerCommand', 'ShellCommand']
