@@ -8,13 +8,21 @@ The only additional thing besides skinning supplied, is the Template traceback
 information.
 """
 import cgi
+import logging
 import sys
 
 from paste.evalexception.middleware import *
 from paste.exceptions.formatter import *
 
+import pylons
+from pylons.middleware import media_path
 from pylons.util import get_prefix
 
+__all__ = []
+
+log = logging.getLogger(__name__)
+
+
 class Supplement(errormiddleware.Supplement):
     """This is a supplement used to display standard WSGI information in
     the traceback.
@@ -23,6 +31,7 @@ class Supplement(errormiddleware.Supplement):
         data = {}
         cgi_vars = data[('extra', 'CGI Variables')] = {}
         wsgi_vars = data[('extra', 'WSGI Variables')] = {}
+        # XXX: Legacy: hiding paste.config
         hide_vars = ['paste.config', 'wsgi.errors', 'wsgi.input',
                      'wsgi.multithread', 'wsgi.multiprocess',
                      'wsgi.run_once', 'wsgi.version',
@@ -41,14 +50,13 @@ class Supplement(errormiddleware.Supplement):
                                        'wsgi.run_once')])
         wsgi_vars['wsgi process'] = self.process_combos[proc_desc]
         wsgi_vars['application'] = self.middleware.application
-        if 'paste.config' in self.environ:
-            data[('extra', 'Configuration')] = \
-                dict(self.environ['paste.config'])
+        data[('extra', 'Configuration')] = pylons.config.current_conf()
             
         # Add any extra sections here
        
         return data
 
+
 class HTMLFormatter(formatter.HTMLFormatter):
     def format_collected_data(self, exc_data):
         general_data = {}
@@ -172,9 +180,11 @@ class HTMLFormatter(formatter.HTMLFormatter):
             # and so the js & CSS are unneeded
             return text, extra_data
 
+
 class InvalidTemplate(Exception):
     pass
 
+
 class PylonsEvalException(EvalException):
 
     def __init__(self, application, global_conf=None, xmlhttp_key=None,
@@ -205,6 +215,16 @@ class PylonsEvalException(EvalException):
         except:
             raise Exception('Invalid template. Please ensure all % signs are properly '
                             'quoted as %% and no extra substitution strings are present.')
+
+    def media(self, environ, start_response):
+        """Handle Pylons specific media content @ /_debug/media/pylons"""
+        if environ['PATH_INFO'].startswith('/pylons'):
+            request.path_info_pop(environ)
+            app = urlparser.StaticURLParser(media_path)
+            return app(environ, start_response)
+        else:
+            return super(self.__class__, self).media(environ, start_response)
+    media.exposed = True
             
     def respond(self, environ, start_response):
         if environ.get('paste.throw_errors'):
@@ -251,7 +271,8 @@ class PylonsEvalException(EvalException):
 
             exc_data = collector.collect_exception(*exc_info)
             #debug_info = PylonsDebugInfo(count, exc_info, exc_data, base_path,
-            debug_info = PylonsDebugInfo(count, exc_info, exc_data, get_prefix(environ),
+            debug_info = PylonsDebugInfo(count, exc_info, exc_data,
+                                         get_prefix(environ, warn=False),
                                          environ, view_uri, error_template)
             assert count not in self.debug_infos
             self.debug_infos[count] = debug_info
@@ -269,15 +290,16 @@ class PylonsEvalException(EvalException):
             return debug_info.content()
 
 
-
 def myghty_html_data(exc_value):
     if hasattr(exc_value, 'htmlformat'):
         return exc_value.htmlformat()[333:-14]
     if hasattr(exc_value, 'mtrace'):
         return exc_value.mtrace.htmlformat()[333:-14]
 
+
 template_error_formatters = [myghty_html_data]
 
+
 try:
     import mako.exceptions
 except ImportError:
@@ -285,10 +307,12 @@ except ImportError:
 else:
     def mako_html_data(exc_value):
         if isinstance(exc_value, (mako.exceptions.CompileException, mako.exceptions.SyntaxException)):
-            return mako.exceptions.html_error_template().render()[610:-16]
+            return mako.exceptions.html_error_template().render(full=False,
+                                                                css=False)
     
     template_error_formatters.insert(0,mako_html_data)
-    
+
+
 class PylonsDebugInfo(DebugInfo):
     def __init__(self, counter, exc_info, exc_data, base_path,
                  environ, view_uri, error_template):
@@ -346,6 +370,7 @@ class PylonsDebugInfo(DebugInfo):
             '</script>\n'
             % (base_path, base_path, base_path, counter))
 
+
 class EvalHTMLFormatter(HTMLFormatter):
 
     def __init__(self, base_path, counter, **kw):
@@ -359,7 +384,7 @@ class EvalHTMLFormatter(HTMLFormatter):
         return (line +
                 '  <a href="#" class="switch_source" '
                 'tbid="%s" onClick="return showFrame(this)">&nbsp; &nbsp; '
-                '<img src="%s/error/img/plus.jpg" border=0 width=9 '
+                '<img src="%s/_debug/media/pylons/img/plus.jpg" border=0 width=9 '
                 'height=9> &nbsp; &nbsp;</a>'
                 % (frame.tbid, self.base_path))
 
@@ -446,6 +471,7 @@ def format_eval_html(exc_data, base_path, counter):
     """ % (short_er, len(short_text_er.splitlines()), short_text_er,
            full_traceback_html), extra_data
 
+
 def format_extra_data_text(exc_data):
     """ Return a text representation of the 'extra_data' dict when one exists """
     extra_data_text = ''
@@ -470,6 +496,7 @@ def format_extra_data_text(exc_data):
         extra_data_text += by_title[title]
     return extra_data_text
 
+
 error_template = '''\
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
@@ -478,10 +505,10 @@ error_template = '''\
  %(head)s
 
 <!-- CSS Imports -->
-<link rel="stylesheet" href="%(prefix)s/error/style/orange.css" type="text/css" media="screen" />
+<link rel="stylesheet" href="%(prefix)s/_debug/media/pylons/style/orange.css" type="text/css" media="screen" />
 
 <!-- Favorite Icons -->
-<link rel="icon" href="%(prefix)s/error/img/icon-16.png" type="image/png" />
+<link rel="icon" href="%(prefix)s/_debug/media/pylons/img/icon-16.png" type="image/png" />
 
 <!-- Mako Styles -->
 <style type="text/css">
@@ -501,7 +528,7 @@ error_template = '''\
 <noscript>
 <div style="border-bottom: 1px solid #808080">
 <div style="border-bottom: 1px solid #404040">
-<table width="100%%" border="0" cellpadding="0" bgcolor="#FFFFE1"><tr><td valign="middle"><img src="%(prefix)s/error/img/warning.gif" alt="Warning" /></td><td>&nbsp;</td><td><span style="padding: 0px; margin: 0px; font-family: Tahoma, sans-serif; font-size: 11px">Warning, your browser does not support JavaScript so you will not be able to use the interactive debugging on this page.</span></td></tr></table>
+<table width="100%%" border="0" cellpadding="0" bgcolor="#FFFFE1"><tr><td valign="middle"><img src="%(prefix)s/_debug/media/pylons/img/warning.gif" alt="Warning" /></td><td>&nbsp;</td><td><span style="padding: 0px; margin: 0px; font-family: Tahoma, sans-serif; font-size: 11px">Warning, your browser does not support JavaScript so you will not be able to use the interactive debugging on this page.</span></td></tr></table>
 </div>
 </div>
 </noscript>
@@ -510,7 +537,7 @@ error_template = '''\
     <a name="top"></a>
     
     <!-- Logo -->
-    <h1 id="logo"><a class="no-underline" href="http://pylonshq.com"><img class="no-border" src="%(prefix)s/error/img/logo.gif" alt="Pylons" title="Pylons"/></a></h1>
+    <h1 id="logo"><a class="no-underline" href="http://pylonshq.com"><img class="no-border" src="%(prefix)s/_debug/media/pylons/img/logo.gif" alt="Pylons" title="Pylons"/></a></h1>
     <p class="invisible"><a href="#content">Skip to content</a></p>
 
     <!-- Main Content -->
@@ -551,6 +578,7 @@ error_template = '''\
 </html>
 '''
 
+
 error_head_template = """
 <!-- 
     This is the Pylons error handler.
@@ -572,7 +600,7 @@ error_head_template = """
 if (document.images)
 {
   pic1= new Image(100,25); 
-  pic1.src="%(prefix)s/error/img/tab-yellow.png"; 
+  pic1.src="%(prefix)s/_debug/media/pylons/img/tab-yellow.png"; 
 }
 
 function switch_display(id) {
@@ -596,6 +624,7 @@ function switch_display(id) {
 </script>
 """
 
+
 error_traceback_template = """
         <div style="float: left; width: 100%%; padding-bottom: 20px;">
         <h1 class="first"><a name="content"></a>Error Traceback</h1>
@@ -610,8 +639,6 @@ error_traceback_template = """
         <b>Extra Features</b>
         <table border="0">
         <tr><td>&gt;&gt;</td><td>Display the lines of code near each part of the traceback</td></tr>
-        <tr><td><img src="%(prefix)s/error/img/plus.jpg" /></td><td>Show a debug prompt to allow you to directly debug the code at the traceback</td></tr>
+        <tr><td><img src="%(prefix)s/_debug/media/pylons/img/plus.jpg" /></td><td>Show a debug prompt to allow you to directly debug the code at the traceback</td></tr>
         </table>
         </div>%(repost_button)s"""
-
-__all__ = []
