@@ -163,7 +163,7 @@ class PylonsBaseWSGIApp(object):
         environ['pylons.routes_dict'] = match
         controller = match.get('controller')
         if not controller:
-            return None
+            return
 
         if self.log_debug:
             log.debug("Resolved URL to controller: %r", controller)
@@ -224,8 +224,7 @@ class PylonsBaseWSGIApp(object):
             return controller(**match)
         
         # If it's a class, instantiate it
-        if not hasattr(controller, '__class__') or \
-            getattr(controller, '__class__') == type:
+        if inspect.isclass(controller):
             if log_debug:
                 log.debug("Controller appears to be a class, instantiating")
             controller = controller()
